import os
import glob
import click
import root_pandas
import pandas as pd

from typing import Optional, List
from multiprocessing.pool import Pool

from analysistools.utilities import PathType


def convert_helper(filename: PathType, out_path: PathType) -> None:
    print(f"Start conversion {str(filename)}")
    df = root_pandas.read_root(filename)  # type: pd.DataFrame
    fname_wo_extension = str(filename).rstrip(".root")  # type: str

    if out_path is not None:
        fout = os.path.join(out_path, f"{os.path.basename(fname_wo_extension)}.hdf")  # type: PathType
    else:
        fout = f"{fname_wo_extension}.hdf"

    df.astype("float32").to_hdf(fout, key="variables", format="table")
    print(f"Finished conversion of {filename}")
    print(f"Output has benn written to {fout}")


@click.command()
@click.option(
    "--num_cpu",
    "-n",
    type=int,
    help="Number of parallel jobs",
)
@click.option(
    "--output_path",
    "-o",
    type=click.Path(),
    default=None,
    help="Path to an alternative output directory",
)
def root2hdf(num_cpu: int = 1, output_path: Optional[PathType] = None) -> None:
    cwd = os.getcwd()  # type: PathType
    root_files = glob.glob(os.path.join(cwd, "*.root"))  # type: List[str]

    if output_path is not None:
        if not os.path.exists(output_path):
            os.makedirs(output_path)

    print(f"Found {len(root_files)} ROOT file(s) in the directory {cwd}:")
    for file in root_files:
        print(f"\t{file}")

    print(f"Starting conversion process with {num_cpu} Core(s)")

    args = [(f_name, output_path) for f_name in root_files]

    with Pool(processes=num_cpu) as pool:
        pool.starmap(convert_helper, args)

    print("Finished conversion process.")
