import pandas as pd

from typing import Optional, List


def uproottree2df(tree, branches: Optional[List[str]] = None) -> pd.DataFrame:
    """
    Take an uproot tree and read it. Return pandas dataframe.
    Should work with both uproot 4 and earlier versions. In the latter case
    a warning statement is printed.

    :param: branches: List of branches to read
    """
    try:
        return tree.arrays(library="pd", expressions=branches)
    except AttributeError:
        print("Using uproot4 syntax failed. Trying deprecated uproot < 4 syntax as a fallback.")
        return tree.pandas.df(branches=branches)
    except TypeError:
        print("Using uproot4 syntax failed. Trying deprecated uproot < 4 syntax as a fallback.")
        return tree.pandas.df(branches=branches)
