#ifndef _Corrections
#define _Corrections

#include <algorithm> 
#include <vector>
#include "TMath.h"
#include "TVectorD.h"
#include "TMatrixD.h"
#include "TRandom2.h"
#include "TEnv.h"
#include "utils/Utils.h"
#include "TheoryClass.h"
#include "LepCorrections/LepEff.h"
#include "LepCorrections/LepFake.h"
#include "LepCorrections/KPiEff.h"
#include "SemiLepDssWeights2D.h"
#include "SemiLepWeights2D.h"

class Corrections{

 private:

  TheoryClass *_FFtheo;
  SemiLepDssWeights2D *_SemiLepDss;
  SemiLepWeights2D *_SemiLepD;
  LepEff *_lepEffElec, *_lepEffMu;
  LepFake *_lepFake;
  KPiEff *_KPiEff;
  float _rhoDs_err, _R1_err, _R2_err;
  float _slowPiWeights[24][18];

  TString _source;
  int _paramNo, _NsigmaFF;

  TRandom2 *_rand;
  UInt_t _seed;
  float _systVar;

 public:

  Corrections(TString syst_source = "none", int no_variedParam = 0, UInt_t seed = 0, int NsigmaFF = 1);
  // possibility to vary B->D*lnu FF by more than 1 sigma
  ~Corrections();

  float BR(float lclass, float dclass, float true_Dstar_channel, float true_D_channel);

  float FF(float lclass, float dclass, float true_w, float true_cv, float true_cl, float true_chi);
  float FFDs_christian(int lclass, int dclass, float true_q2, float true_pl);
  float DssFF(int dclass, float w_true, float costh_true, int iVarBin=0);
  float DFF( int dclass, float q2_true, float plep_true, int iVarBin=0);

  float slowPion(int exp, int truthMatching, int Dscharge, int Dcharge, float pPi);

  float lepEff(int fl_lep, int exp_no, float theta, float plep);
  float lepFake(int fl_lep, float theta, float plep);
  float kPiEff(std::vector<int> *trkSep_list, std::vector<float> *trkTh_list, std::vector<float> *trkP_list, int exp);

  float tagging(float b_mode, float NB);


  //  ______________________________________________________________________________
  // | Variations of 1 sigma to evaluate systematics                                |
  // |______________________________________________________________________________|
  // | Correction: parameter no. (sign is sign of variation)                        | 
  // |                                                                              |
  // | BR corrections:                                                              |
  // |--------------------------------------------------                            |
  // | "Ddecay":    1="KPi",     2="K2Pi",    3="K3Pi",    4="KPiPi0"               |
  // | "Dsdecay":   1="D0Pi",    2="D0Pi0",   3="DPi0",    4="D0gamma", 5="Dgamma"  |
  // | "BtoDslnu":  1= "Bch",    2="B0"                                             |
  // | "BtoDsslnu": 1="BchDss1", 2="BchDss2", 3="BchDss3", 4="BchDss4"              |
  // |              5="B0Dss1",  6=B0Dss2",   7="B0Dss3",  8="B0Dss4"               |
  // | "BtoDlnu":   1="BchD",    2="B0D"                                            |
  // |                                                                              |                
  // | FF parameters                                                                |
  // |--------------------------------------------------                            |
  // | "DsFF":  1, 2, 3                                                             |
  // | "DssFF": 1, 2, 3                                                             |
  // | "DFF":   1                                                                   |
  // |                                                                              |
  // | Reco efficiencies                                                            |
  // |--------------------------------------------------                            |
  // | "slowPi":  1,..., 6 = Pi+-/SVD1,                                             |
  // |            7,...,12 = Pi+-/SVD2,                                             |
  // |           13,...,18 = Pi0 /SVD1,                                             |
  // |           19,...,24 = Pi0 /SVD2                                              |
  // |                                                                              |
  // | "lepEff_el", "lepEff_mu", "lepFake", "KPiEff", "tagEff" (via toys: give seed)|
  // |______________________________________________________________________________|





 private:

  float BR_D(float true_D_channel);
  float BR_Dstar(float true_Dstar_channel);
  float BR_B(float true_Dstar_channel);
  float BR_bkg(float lclass, float dclass);

  TVectorD calcVariatedFF(int variedPar);

  int sign(float n);

};


#endif
