"""
Slow Pion correction table extension by calculating variations of the correction factors from the related uncertainties.

Calculates the variations of the slow pion correction table.

Sources:
General: https://belle.kek.jp/secured/wiki/doku.php?id=software:systematics
Data: https://belle.kek.jp/secured/belle_note/gn1176/
Belle Note: https://belle.kek.jp/secured/belle_note/gn1176/bn1176.pdf
Talk: https://kds.kek.jp/event/6029/contributions/138416/attachments/107716/127702/BAM-2010-11-15-WolfgangDungel.pdf

Felix Metzner 2023
"""

import os
import pandas as pd

from typing import List
from dataclasses import dataclass

from analysistools.utilities import PathType


__all__ = [
    "SlowPionTablePreparer"
]


@dataclass(frozen=True)
class SlowPionTablesContainer:
    pi_plus_old_svd: pd.DataFrame
    pi_plus_new_svd: pd.DataFrame
    pi_zero_old_svd: pd.DataFrame
    pi_zero_new_svd: pd.DataFrame

    @staticmethod
    def load_tables() -> "SlowPionTablesContainer":

        new_container = SlowPionTablesContainer(
            pi_plus_old_svd=pi_plus_old_svd_table,
            pi_plus_new_svd=pi_plus_new_svd_table,
            pi_zero_old_svd=pi_zero_old_svd_table,
            pi_zero_new_svd=pi_zero_new_svd_table,
        )

        return new_container


class SlowPionTablePreparer:
    number_of_variations: int = 12

    def __init__(self) -> None:
        self.weight_name: str = "slow_pi"

        self.table_name: str = "slowPionAllWeights"
        self.base_path: PathType = os.path.dirname(os.path.abspath(__file__))

        self.columns = self.get_slow_pion_table_columns()

        self.momentum_column: str = "momentum_lower"
        self.charge_column: str = "charge"
        self.exp_column: str = "exp_lower"

        self.momentum_bins: List[float] = self.get_momentum_bins()

    def load_base_tables(self) -> SlowPionTablesContainer:
        pass

    def create_full_table_with_variations(self) -> pd.DataFrame:
        pass

    def get_varied_table(self) -> pd.DataFrame:
        pass


def run_slow_pion_table_preparer() -> None:
    pass


if __name__ == "__main__":
    run_slow_pion_table_preparer()
