"""
Unit Test for the ThreeVector module
Max Welsch 2018.
"""

import unittest

import numpy as np
from analysistools.physics_vectors.threevector import ThreeVector


class TestThreeVector(unittest.TestCase):
    def setUp(self):
        self.v1 = ThreeVector(np.array([[1, 1, 1], [1, 0, 0]]))
        self.v2 = ThreeVector(np.array([[1, 2, 3], [0, 1, 0]]))

    def test_as_array(self):
        expected = np.array([[1, 1, 1], [1, 0, 0]])
        np.testing.assert_equal(self.v1.as_array, expected)

    def test_angle(self):
        expected = np.array([0.3876, 1.5708])
        np.testing.assert_almost_equal(self.v1.Angle(self.v2), expected, decimal=3)

    def test_dot(self):
        expected = np.array([6, 0])
        np.testing.assert_equal(self.v1.Dot(self.v2), expected)

    def test_mag2(self):
        expected = np.array([3, 1])
        np.testing.assert_equal(self.v1.Mag2(), expected)

    def test_mag(self):
        expected = np.sqrt(np.array([3, 1]))
        np.testing.assert_equal(self.v1.Mag(), expected)

    def test_add(self):
        expected = np.array([[2, 3, 4], [1, 1, 0]])
        v = self.v1 + self.v2
        np.testing.assert_equal(v.as_array, expected)

    def test_sub(self):
        expected = np.array([[0, -1, -2], [1, -1, 0]])
        v = self.v1 - self.v2
        np.testing.assert_equal(v.as_array, expected)

    def test_mul(self):
        expected = np.array([6, 0])
        res = self.v1 * self.v2
        np.testing.assert_equal(res, expected)

    def test_cross(self):
        cross = self.v1.Cross(self.v2)
        expected = np.array([[1, -2, 1], [0, 0, 1]])
        np.testing.assert_equal(cross.as_array, expected)

    def test_unit(self):
        unit = self.v1.Unit()
        expected = np.array([[1 / np.sqrt(3), 1 / np.sqrt(3), 1 / np.sqrt(3)], [1, 0, 0]])
        np.testing.assert_almost_equal(unit.as_array, expected, decimal=3)

    def test_perp(self):
        perp = self.v1.Perp()
        expected = np.sqrt(np.array([2, 1]))
        np.testing.assert_equal(perp, expected)

    def test_scale(self):
        self.v1.Scale(5)
        np.testing.assert_equal(self.v1.as_array, np.array([[5, 5, 5], [5, 0, 0]]))
