import unittest

import numpy as np

from analysistools.offline_analysis.histogram import Histogram


class TestHistogram(unittest.TestCase):
    def setUp(self):
        self.data = np.array([0.2, 0.5, 0.6, 1.5, 1.3, 2.4])
        self.weights = np.array([0.5, 0.5, 1.0, 1, 0.5, 0.8])
        self.range = (0, 5)

        self.simple_hist_w = Histogram(3, (0.0, 3.0))
        self.simple_hist_w.fill(self.data, self.weights)

        self.simple_hist = Histogram(3, (0.0, 3.0))
        self.simple_hist.fill(self.data)

        self.simple_hist_w_normed = Histogram(3, (0.0, 3.0), normed=True)
        self.simple_hist_w_normed.fill(self.data, self.weights)

        self.simple_hist_normed = Histogram(3, (0.0, 3.0), normed=True)
        self.simple_hist_normed.fill(self.data)

    def test_bin_counts(self):
        np.testing.assert_almost_equal(self.simple_hist_w.bin_counts, np.array([2.0, 1.5, 0.8]))
        np.testing.assert_almost_equal(self.simple_hist.bin_counts, np.array([3.0, 2, 1]))

    def test_bin_counts_normed(self):
        np.testing.assert_almost_equal(self.simple_hist_w_normed.bin_counts, np.array([0.465, 0.349, 0.186]), decimal=3)

        np.testing.assert_almost_equal(self.simple_hist_normed.bin_counts, np.array([0.500, 1 / 3.0, 1 / 6.0]), decimal=3)

    def test_bin_errors(self):
        np.testing.assert_almost_equal(self.simple_hist_w.bin_errors, np.array([1.225, 1.118, 0.8]), decimal=3)

        np.testing.assert_almost_equal(self.simple_hist.bin_errors, np.sqrt(np.array([3.0, 2, 1])), decimal=3)

    def test_bin_errors_normed(self):
        np.testing.assert_almost_equal(
            self.simple_hist_w_normed.bin_errors, (1 / 4.3) * np.array([1.225, 1.118, 0.8]), decimal=3
        )

        np.testing.assert_almost_equal(
            self.simple_hist_normed.bin_errors, (1 / 6) * np.sqrt(np.array([3.0, 2, 1])), decimal=3
        )

    def test_additional_fill(self):
        self.simple_hist_w.fill(np.array([0.5, 2.3]), np.array([1.2, 1.5]))
        np.testing.assert_almost_equal(self.simple_hist_w.bin_counts, np.array([3.2, 1.5, 2.3]))
        np.testing.assert_almost_equal(self.simple_hist_w.bin_errors, np.array([1.715, 1.118, 1.7]), decimal=3)

        self.simple_hist.fill(np.array([0.5, 2.3]))
        np.testing.assert_almost_equal(self.simple_hist.bin_counts, np.array([4, 2, 2]))
        np.testing.assert_almost_equal(self.simple_hist.bin_errors, np.sqrt(np.array([4, 2, 2])), decimal=3)

        self.simple_hist_w_normed.fill(np.array([0.5, 2.3]), np.array([1.2, 1.5]))
        np.testing.assert_almost_equal(self.simple_hist_w_normed.bin_counts, (1 / 7) * np.array([3.2, 1.5, 2.3]))
        np.testing.assert_almost_equal(
            self.simple_hist_w_normed.bin_errors, (1 / 7) * np.array([1.715, 1.118, 1.7]), decimal=3
        )

        self.simple_hist_normed.fill(np.array([0.5, 2.3]))
        np.testing.assert_almost_equal(self.simple_hist_normed.bin_counts, (1 / 8) * np.array([4, 2, 2]))
        np.testing.assert_almost_equal(
            self.simple_hist_normed.bin_errors, (1 / 8) * np.sqrt(np.array([4, 2, 2])), decimal=3
        )

    def test_bin_edges(self):
        bin_width = self.simple_hist.bin_width
        np.testing.assert_equal(bin_width, 1.0)
